@@ -191,6 +191,40 @@ Possible values:
 Related options:
     * image_property_quota
 
+""")),
+    cfg.StrOpt('hashing_algorithm',
+               default='sha512',
+               help=_(""""
+Secure hashing algorithm used for computing the 'os_hash_value' property.
+
+This option configures the Glance "multihash", which consists of two
+image properties: the 'os_hash_algo' and the 'os_hash_value'.  The
+'os_hash_algo' will be populated by the value of this configuration
+option, and the 'os_hash_value' will be populated by the hexdigest computed
+when the algorithm is applied to the uploaded or imported image data.
+
+The value must be a valid secure hash algorithm name recognized by the
+python 'hashlib' library.  You can determine what these are by examining
+the 'hashlib.algorithms_available' data member of the version of the
+library being used in your Glance installation.  For interoperability
+purposes, however, we recommend that you use the set of secure hash
+names supplied by the 'hashlib.algorithms_guaranteed' data member because
+those algorithms are guaranteed to be supported by the 'hashlib' library
+on all platforms.  Thus, any image consumer using 'hashlib' locally should
+be able to verify the 'os_hash_value' of the image.
+
+The default value of 'sha512' is a performant secure hash algorithm.
+
+If this option is misconfigured, any attempts to store image data will fail.
+For that reason, we recommend using the default value.
+
+Possible values:
+    * Any secure hash algorithm name recognized by the Python 'hashlib'
+      library
+
+Related options:
+    * None
+
 """)),
     cfg.IntOpt('image_member_quota', default=128,
                help=_("""
@@ -371,14 +405,26 @@ Related options:
 """)),
     # NOTE(flaper87): The policy.json file should be updated and the locaiton
     # related rules set to admin only once this option is finally removed.
+    # NOTE(rosmaita): Unfortunately, this option is used to gate some code
+    # paths; if the location related policies are set admin-only, then no
+    # normal users can save or retrieve image data.
     cfg.BoolOpt('show_multiple_locations', default=False,
                 deprecated_for_removal=True,
-                deprecated_reason=_('This option will be removed in the Pike '
-                                    'release or later because the same '
-                                    'functionality can be achieved with '
-                                    'greater granularity by using policies. '
-                                    'Please see the Newton '
-                                    'release notes for more information.'),
+                deprecated_reason=_('Use of this option, deprecated since '
+                                    'Newton, is a security risk and will be '
+                                    'removed once we figure out a way to '
+                                    'satisfy those use cases that currently '
+                                    'require it.  An earlier announcement '
+                                    'that the same functionality can be '
+                                    'achieved with greater granularity by '
+                                    'using policies is incorrect.  You cannot '
+                                    'work around this option via policy '
+                                    'configuration at the present time, '
+                                    'though that is the direction we believe '
+                                    'the fix will take.  Please keep an eye '
+                                    'on the Glance release notes to stay up '
+                                    'to date on progress in addressing this '
+                                    'issue.'),
                 deprecated_since='Newton',
                 help=_("""
 Show all image locations when returning an image.
@@ -395,6 +441,8 @@ NOTES:
       image locations can sometimes include credentials. Hence, this
       is set to ``False`` by default. Set this to ``True`` with
       EXTREME CAUTION and ONLY IF you know what you are doing!
+    * See https://wiki.openstack.org/wiki/OSSN/OSSN-0065 for more
+      information.
     * If an operator wishes to avoid showing any image location(s)
       to the user, then both this option and
       ``show_image_direct_url`` MUST be set to ``False``.
@@ -454,50 +502,6 @@ Possible values:
 Related options:
     * None
 
-""")),
-    # NOTE(nikhil): Even though deprecated, the configuration option
-    # ``enable_v1_api`` is set to True by default on purpose. Having it enabled
-    # helps the projects that haven't been able to fully move to v2 yet by
-    # keeping the devstack setup to use glance v1 as well. We need to switch it
-    # to False by default soon after Newton is cut so that we can identify the
-    # projects that haven't moved to v2 yet and start having some interesting
-    # conversations with them. Switching to False in Newton may result into
-    # destabilizing the gate and affect the release.
-    cfg.BoolOpt('enable_v1_api',
-                default=True,
-                deprecated_reason=_DEPRECATE_GLANCE_V1_MSG,
-                deprecated_since='Newton',
-                help=_("""
-Deploy the v1 OpenStack Images API.
-
-When this option is set to ``True``, Glance service will respond to
-requests on registered endpoints conforming to the v1 OpenStack
-Images API.
-
-NOTES:
-    * If this option is enabled, then ``enable_v1_registry`` must
-      also be set to ``True`` to enable mandatory usage of Registry
-      service with v1 API.
-
-    * If this option is disabled, then the ``enable_v1_registry``
-      option, which is enabled by default, is also recommended
-      to be disabled.
-
-    * This option is separate from ``enable_v2_api``, both v1 and v2
-      OpenStack Images API can be deployed independent of each
-      other.
-
-    * If deploying only the v2 Images API, this option, which is
-      enabled by default, should be disabled.
-
-Possible values:
-    * True
-    * False
-
-Related options:
-    * enable_v1_registry
-    * enable_v2_api
-
 """)),
     cfg.BoolOpt('enable_v2_api',
                 default=True,
@@ -523,20 +527,12 @@ NOTES:
       option, which is enabled by default, is also recommended
       to be disabled.
 
-    * This option is separate from ``enable_v1_api``, both v1 and v2
-      OpenStack Images API can be deployed independent of each
-      other.
-
-    * If deploying only the v1 Images API, this option, which is
-      enabled by default, should be disabled.
-
 Possible values:
     * True
     * False
 
 Related options:
     * enable_v2_registry
-    * enable_v1_api
 
 """)),
     cfg.BoolOpt('enable_v1_registry',
@@ -544,25 +540,7 @@ Related options:
                 deprecated_reason=_DEPRECATE_GLANCE_V1_MSG,
                 deprecated_since='Newton',
                 help=_("""
-Deploy the v1 API Registry service.
-
-When this option is set to ``True``, the Registry service
-will be enabled in Glance for v1 API requests.
-
-NOTES:
-    * Use of Registry is mandatory in v1 API, so this option must
-      be set to ``True`` if the ``enable_v1_api`` option is enabled.
-
-    * If deploying only the v2 OpenStack Images API, this option,
-      which is enabled by default, should be disabled.
-
-Possible values:
-    * True
-    * False
-
-Related options:
-    * enable_v1_api
-
+                    DEPRECATED FOR REMOVAL
 """)),
     cfg.BoolOpt('enable_v2_registry',
                 default=True,
@@ -700,32 +678,8 @@ Possible values:
 
 Related options:
     * [task]/work_dir
-    * [DEFAULT]/enable_image_import (*deprecated*)
 
 """)),
-    cfg.BoolOpt('enable_image_import',
-                default=True,
-                deprecated_for_removal=True,
-                deprecated_reason=_("""
-This option is deprecated for removal in Rocky.
-
-It was introduced to make sure that the API is not enabled
-before the '[DEFAULT]/node_staging_uri' is defined and is
-long term redundant."""),
-                deprecated_since='Pike',
-                help=_("""
-Enables the Image Import workflow introduced in Pike
-
-As '[DEFAULT]/node_staging_uri' is required for the Image
-Import, it's disabled per default in Pike, enabled per
-default in Queens and removed in Rocky. This allows Glance to
-operate with previous version configs upon upgrade.
-
-Setting this option to False will disable the endpoints related
-to Image Import Refactoring work.
-
-Related options:
-    * [DEFAULT]/node_staging_uri""")),
     cfg.ListOpt('enabled_import_methods',
                 item_type=cfg.types.String(quotes=True),
                 bounds=True,
@@ -736,8 +690,7 @@ List of enabled Image Import Methods
 Both 'glance-direct' and 'web-download' are enabled by default.
 
 Related options:
-    * [DEFAULT]/node_staging_uri
-    * [DEFAULT]/enable_image_import""")),
+    * [DEFAULT]/node_staging_uri""")),
 ]
 
 CONF = cfg.CONF
