@@ -57,8 +57,13 @@ def _calc_required_size(context, image, locations):
             size_from_backend = None
 
             try:
-                size_from_backend = store.get_size_from_backend(
-                    location['url'], context=context)
+                if CONF.enabled_backends:
+                    size_from_backend = store.get_size_from_uri_and_backend(
+                        location['url'], location['metadata'].get('backend'),
+                        context=context)
+                else:
+                    size_from_backend = store.get_size_from_backend(
+                        location['url'], context=context)
             except (store.UnknownScheme, store.NotFound):
                 pass
             except store.BadStoreUri:
@@ -160,6 +165,14 @@ class QuotaImageTagsProxy(object):
         return self.tags.__len__(*args, **kwargs)
 
     def __getattr__(self, name):
+        # Protect against deepcopy, which calls getattr. __getattr__
+        # is only called when an attribute is not "normal", so when
+        # self.tags is called, this is not.
+        if name == 'tags':
+            try:
+                return self.__getattribute__('tags')
+            except AttributeError:
+                return None
         return getattr(self.tags, name)
 
 
@@ -293,18 +306,16 @@ class ImageProxy(glance.domain.proxy.Image):
         super(ImageProxy, self).__init__(image)
         self.orig_props = set(image.extra_properties.keys())
 
-    def set_data(self, data, size=None):
+    def set_data(self, data, size=None, backend=None):
         remaining = glance.api.common.check_quota(
             self.context, size, self.db_api, image_id=self.image.image_id)
         if remaining is not None:
             # NOTE(jbresnah) we are trying to enforce a quota, put a limit
             # reader on the data
-            data = utils.LimitingReader(data, remaining)
-        try:
-            self.image.set_data(data, size=size)
-        except exception.ImageSizeLimitExceeded:
-            raise exception.StorageQuotaFull(image_size=size,
-                                             remaining=remaining)
+            data = utils.LimitingReader(
+                data, remaining, exception_class=exception.StorageQuotaFull)
+
+        self.image.set_data(data, size=size, backend=backend)
 
         # NOTE(jbresnah) If two uploads happen at the same time and neither
         # properly sets the size attribute[1] then there is a race condition
