@@ -31,7 +31,6 @@ import six
 from six.moves import http_client as http
 import webob
 
-from glance.api.v1 import router as router_v1
 from glance.api.v2 import router as router_v2
 from glance.common import exception
 from glance.common import utils
@@ -50,7 +49,8 @@ class RequestTest(test_utils.BaseTestCase):
         def returns_some_locales(*args, **kwargs):
             return all_locales
 
-        self.stubs.Set(localedata, 'locale_identifiers', returns_some_locales)
+        self.mock_object(localedata, 'locale_identifiers',
+                         returns_some_locales)
 
         # Override gettext.find to return other than None for some languages.
         def fake_gettext_find(lang_id, *args, **kwargs):
@@ -63,7 +63,7 @@ class RequestTest(test_utils.BaseTestCase):
                 return found_ret
             return None
 
-        self.stubs.Set(gettext, 'find', fake_gettext_find)
+        self.mock_object(gettext, 'find', fake_gettext_find)
 
     def test_content_range(self):
         request = wsgi.Request.blank('/tests/123')
@@ -184,16 +184,20 @@ class RequestTest(test_utils.BaseTestCase):
         req = wsgi.Request.blank('/', headers={'Accept-Language': 'unknown'})
         self.assertIsNone(req.best_match_language())
 
-    @mock.patch.object(webob.acceptparse.AcceptLanguage, 'best_match')
-    def test_best_match_language_unknown(self, mock_best_match):
-        # Test that we are actually invoking language negotiation by webop
+    @mock.patch.object(webob.acceptparse.AcceptLanguageValidHeader, 'lookup')
+    def test_best_match_language_unknown(self, mock_lookup):
+        # Test that we are actually invoking language negotiation by WebOb
         request = wsgi.Request.blank('/')
         accepted = 'unknown-lang'
         request.headers = {'Accept-Language': accepted}
 
-        mock_best_match.return_value = None
+        # Bug #1765748: see comment in code in the function under test
+        # to understand why this is the correct return value for the
+        # webob 1.8.x mock
+        mock_lookup.return_value = 'fake_LANG'
 
         self.assertIsNone(request.best_match_language())
+        mock_lookup.assert_called_once()
 
         # If Accept-Language is missing or empty, match should be None
         request.headers = {'Accept-Language': ''}
@@ -204,24 +208,6 @@ class RequestTest(test_utils.BaseTestCase):
     def test_http_error_response_codes(self):
         sample_id, member_id, tag_val, task_id = 'abc', '123', '1', '2'
 
-        """Makes sure v1 unallowed methods return 405"""
-        unallowed_methods = [
-            ('/images', ['PUT', 'DELETE', 'HEAD', 'PATCH']),
-            ('/images/detail', ['POST', 'PUT', 'DELETE', 'PATCH']),
-            ('/images/%s' % sample_id, ['POST', 'PATCH']),
-            ('/images/%s/members' % sample_id,
-                ['POST', 'DELETE', 'HEAD', 'PATCH']),
-            ('/images/%s/members/%s' % (sample_id, member_id),
-                ['POST', 'HEAD', 'PATCH']),
-        ]
-        api = test_utils.FakeAuthMiddleware(router_v1.API(routes.Mapper()))
-        for uri, methods in unallowed_methods:
-            for method in methods:
-                req = webob.Request.blank(uri)
-                req.method = method
-                res = req.get_response(api)
-                self.assertEqual(http.METHOD_NOT_ALLOWED, res.status_int)
-
         """Makes sure v2 unallowed methods return 405"""
         unallowed_methods = [
             ('/schemas/image', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
@@ -342,7 +328,7 @@ class ResourceTest(test_utils.BaseTestCase):
             if isinstance(obj, wsgi.JSONResponseSerializer):
                 raise webob.exc.HTTPForbidden()
 
-        self.stubs.Set(wsgi.Resource, 'dispatch', dispatch)
+        self.mock_object(wsgi.Resource, 'dispatch', dispatch)
 
         request = wsgi.Request.blank('/')
 
@@ -361,7 +347,7 @@ class ResourceTest(test_utils.BaseTestCase):
         def dispatch(self, obj, action, *args, **kwargs):
             raise Exception("test exception")
 
-        self.stubs.Set(wsgi.Resource, 'dispatch', dispatch)
+        self.mock_object(wsgi.Resource, 'dispatch', dispatch)
 
         request = wsgi.Request.blank('/')
 
@@ -392,12 +378,11 @@ class ResourceTest(test_utils.BaseTestCase):
                               resource, request)
         self.assertEqual(message_es, str(e))
 
-    @mock.patch.object(webob.acceptparse.AcceptLanguage, 'best_match')
+    @mock.patch.object(webob.acceptparse.AcceptLanguageValidHeader, 'lookup')
     @mock.patch.object(i18n, 'translate')
-    def test_translate_exception(self, mock_translate, mock_best_match):
-
+    def test_translate_exception(self, mock_translate, mock_lookup):
         mock_translate.return_value = 'No Encontrado'
-        mock_best_match.return_value = 'de'
+        mock_lookup.return_value = 'de'
 
         req = wsgi.Request.blank('/tests/123')
         req.headers["Accept-Language"] = "de"
@@ -603,8 +588,11 @@ class ServerTest(test_utils.BaseTestCase):
                                                 keepalive=False,
                                                 socket_timeout=900)
 
-    def test_number_of_workers(self):
+    def test_number_of_workers_posix(self):
         """Ensure the number of workers matches num cpus limited to 8."""
+        if os.name == 'nt':
+            raise self.skipException("Unsupported platform.")
+
         def pid():
             i = 1
             while True:
