@@ -23,7 +23,6 @@ import time
 
 import fixtures
 from oslo_utils import units
-from oslotest import moxstubout
 import six
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
 from six.moves import range
@@ -282,6 +281,7 @@ class ImageCacheTestCase(object):
         self.assertEqual(['0', '1', '2'],
                          self.cache.get_queued_images())
 
+    @skip_if_disabled
     def test_open_for_write_good(self):
         """
         Test to see if open_for_write works in normal case
@@ -301,6 +301,7 @@ class ImageCacheTestCase(object):
         self.assertFalse(os.path.exists(incomplete_file_path))
         self.assertFalse(os.path.exists(invalid_file_path))
 
+    @skip_if_disabled
     def test_open_for_write_with_exception(self):
         """
         Test to see if open_for_write works in a failure case for each driver
@@ -325,6 +326,7 @@ class ImageCacheTestCase(object):
         self.assertFalse(os.path.exists(incomplete_file_path))
         self.assertTrue(os.path.exists(invalid_file_path))
 
+    @skip_if_disabled
     def test_caching_iterator(self):
         """
         Test to see if the caching iterator interacts properly with the driver
@@ -352,6 +354,7 @@ class ImageCacheTestCase(object):
         self.assertFalse(os.path.exists(incomplete_file_path))
         self.assertFalse(os.path.exists(invalid_file_path))
 
+    @skip_if_disabled
     def test_caching_iterator_handles_backend_failure(self):
         """
         Test that when the backend fails, caching_iter does not continue trying
@@ -375,6 +378,7 @@ class ImageCacheTestCase(object):
         # make sure bad image was not cached
         self.assertFalse(self.cache.is_cached(image_id))
 
+    @skip_if_disabled
     def test_caching_iterator_falloffend(self):
         """
         Test to see if the caching iterator interacts properly with the driver
@@ -403,6 +407,7 @@ class ImageCacheTestCase(object):
         self.assertFalse(os.path.exists(incomplete_file_path))
         self.assertTrue(os.path.exists(invalid_file_path))
 
+    @skip_if_disabled
     def test_gate_caching_iter_good_checksum(self):
         image = b"12345678990abcdefghijklmnop"
         image_id = 123
@@ -418,6 +423,7 @@ class ImageCacheTestCase(object):
         # checksum is valid, fake image should be cached:
         self.assertTrue(cache.is_cached(image_id))
 
+    @skip_if_disabled
     def test_gate_caching_iter_bad_checksum(self):
         image = b"12345678990abcdefghijklmnop"
         image_id = 123
@@ -518,9 +524,7 @@ class TestImageCacheNoDep(test_utils.BaseTestCase):
         def init_driver(self2):
             self2.driver = self.driver
 
-        mox_fixture = self.useFixture(moxstubout.MoxStubout())
-        self.stubs = mox_fixture.stubs
-        self.stubs.Set(image_cache.ImageCache, 'init_driver', init_driver)
+        self.mock_object(image_cache.ImageCache, 'init_driver', init_driver)
 
     def test_get_caching_iter_when_write_fails(self):
 
